/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Helmholtz Centre for Environmental Research - UFZ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 * hal_DRV8711.c
 *
 * Hardware abstraction layer for the BOOST-DRV8711 on MSP-EXP430FR6989
 *
 *  Created on: 07.02.2017
 *      Author: G. Laube
 */

#include <driverlib.h>
#include "hal_LCD.h"
#include "hal_DRV8711.h"

/* Nema11 Motor:
 * max speed = 35 U/min
 * max acceleration = 1.5E6 1/16 steps/s^2
 * gear ratio = 99 1044/2057 : 1
 * original step angle = 1.8 deg
 * -> 360 deg = 200*(99+1044/2057) = 19901.51 steps
 * -> 20 deg = 20/1.8*(99+1044/2057) = 1105.64 steps
 * 18*1106 = 19908 steps = 360.117 deg
 * -> 1106 steps/20 deg sufficiently exact
 *
 * 35 U/min * 19901.51 steps/U /60 s/min = 11609 steps/s
 * -> 32768 Hz / 11609 ~= minimum 3 aclk clock cycles / full step
 * -> choose minimum of 10 clock cycles / micro step
 */

/* Nema23 Motor:
 * max speed = 25 U/min
 * max acceleration = 1E6 1/16 steps/s^2
 * gear ratio = 76 49/64 : 1
 * original step angle = 1.8 deg
 * -> 360 deg = 200*(77+49/64) = 15353.125 steps
 * -> 20 deg = 20/1.8*(77+49/64) = 852.95 steps
 * -> 853 steps/20 deg
 *
 * 25 U/min * 15353.125 steps/U /60 s/min = 6397 steps/s
 * -> 32768 Hz / 6397 ~= minimum 5 aclk clock cycles / full step
 * -> choose minimum of 10 clock cycles / micro step
 */

/*Chopping Current considerations page 16f
 * Nema11- Motor:   max: 5.88Nm at 0.67A
 *              allowed: 3.14Nm ~ 0.35 A
 * I_FS = 0.35A
 *
 * I_FS = 2.75V x TORQUE / 256 / ISGAIN / R_ISENSE
 *
 * R_ISENSE = 0.05 ?
 * ISGAIN = 5/10/20/40
 *
 * select: ISGAIN = 40 = 0b11
 * => TORQUE = 65
 * TORTQUE_MOTOR_MAX = 125
 *
 *Nema23- Motor:   max: 94.1Nm at 2.8A
 *             allowed: 23.5Nm ~ 0.7 A
 * I_FS = 0.7A
 *
 * I_FS = 2.75V x TORQUE / 256 / ISGAIN / R_ISENSE
 * TORQUE = IFS * 256 * ISGAIN * R_ISENSE / 2.75V
 * R_ISENSE = 0.05 Ohm
 * ISGAIN = 5/10/20/40
 *
 * select: ISGAIN = 40 = 0b11
 * => TORQUE = 130
 * ISGAIN_MOTOR_MAX = 10 = 0b01
 * TORTQUE_MOTOR_MAX = 130
 *
 */

/*BLANK considerations page 19f
 * TBLANK = 0x08 (taken from Boostpack manual screenshot p. 12)
 * t_blank = TBLANK * 20ns but at least 1mus
 * t_blank = 1.00 mus
 * TBLANK = 0x32 (= 50)
 * ABT = 1 useful only if microstepping = 1/8 ... 1/256
 * but it doesn't hurt, so ABT = 0b1
 */

/*DRIVE considerations page 22
 * Total Gate Charge = 14 nC, (Gate Charge Gate-to-Drain = 2.3 nC)
 * Desired rise time = 400ns ???
 * IDRIVEP > Q/RT = 35 mA ~ 50mA
 * IDRIVEN = 2xIDRIVEP = 100mA
 *
 * TDRIVEP = TDRIVEN > 2x RT = 1mus
 */

/*TOFF considerations page 22
 * It is recommended to leave DTIME as its default value unless the stepping speed is very high.
 * => DTIME = 850ns = 0b11
 * Q< 5 x (2xDTIME + TBLANK + TOFF)
 * => TOFF > Q/5 - 2xDTIME - TBLANK
 * TOFF > 14/5 - 2x0.85 -1.00
 * TOFF > 0.1 mus = 100ns
 * select TOFF = 2mus = 0x04
 */

/*OCP considerations page 24
 * R_DS < 20 m? (CSD88537ND datasheet)
 * I_DS max ~ 1 A
 *
 * Overcurrent protection voltage:
 * U_DS = I_DS x R_DS
 * U_DS = 20 mV
 * OCPTH = 20 mV => 250 mV = 0b00
 * minimum OCPTH is far too high, so at least turn of fast:
 * OCPDEG = 1 mus = 0b00
 */

/*Stall considerations
 * No stall handling routine implemented, so all stall detection parameters to max:
 * EXSTALL = 0b0 (internal stall detection)
 * SMPLTH = 0b111 (back EMF sample = 1000 mus)
 * SDTHR = 0xFF (stall detection threshold)
 * SDCNT = 0b11 (STALLn asserted after 8 steps)
 * VDIV = 0b11 (Back EMF is divided by 4)
 *
 * */

void init_DRV8711(void)
{
    // should be in sleep already, but better doublecheck
    // driver is still capable of receiving SPI signals in sleep
    //DRV8711_enter_sleep();
    //DRV8711_reset();

    microstep_mode = 0x04;
    microsteps = 1 << microstep_mode;

    // Set Default Register Settings
    // CTRL Register
    CTRL_REG.Address  = 0x00; // CTRL register Address
    CTRL_REG.DTIME    = 0x03; // 850ns
    CTRL_REG.ISGAIN   = 0x01; // 10
    CTRL_REG.EXSTALL  = 0x01; // external stall detection
    CTRL_REG.MODE     = microstep_mode; // 1/8 step mode
    CTRL_REG.RSTEP    = 0x00; // no software stepping
    CTRL_REG.RDIR     = 0x01; // inverse dir-pin interpretation
    CTRL_REG.ENBL     = 0x01; // enable motor

    // TORQUE Register
    TORQUE_REG.Address = 0x01; // TORQUE register Address
    TORQUE_REG.SMPLTH  = 0x07; // stall detection threshold = 1ms
    TORQUE_REG.TORQUE  = 0x82; // torque 0x82 = 130 = 0.7A/2.8A, depending on ISGAIN=0x03/0x01

    // OFF Register
    OFF_REG.Address   = 0x02;  // OFF Register Address
    OFF_REG.PWMMODE   = 0x00;  // internal indexer
    OFF_REG.TOFF      = 0x04;  // 2mus off time

    // BLANK Register
    BLANK_REG.Address = 0x03;  // BLANK register Address
    BLANK_REG.ABT     = 0x01;  // enable adaptive blanking time
    BLANK_REG.TBLANK  = 0x32;  // 1mus time before checking I

    // DECAY Register
    DECAY_REG.Address = 0x04;  // DECAY Register Address
    DECAY_REG.DECMOD  = 0x05;  // auto mixed decay at all times
    DECAY_REG.TDECAY  = 0x02;  // 2 mus should be ignored in auto decay

    // STALL Register
    STALL_REG.Address = 0x05;  // STALL Register Address
    STALL_REG.VDIV    = 0x03;  // Back EMF is divided by 4
    STALL_REG.SDCNT   = 0x03;  // STALLn asserted after 8 steps
    STALL_REG.SDTHR   = 0x00;  // back EMF voltage level at min

    // DRIVE Register
    DRIVE_REG.Address = 0x06;  // DRIVE Register Address
    DRIVE_REG.IDRIVEP = 0x00;  // 50 mA
    DRIVE_REG.IDRIVEN = 0x00;  // 100 mA
    DRIVE_REG.TDRIVEP = 0x02;  // 1 mus
    DRIVE_REG.TDRIVEN = 0x02;  // 1 mus
    DRIVE_REG.OCPDEG  = 0x00;  // 1 mus
    DRIVE_REG.OCPTH   = 0x00;  // 250mV

    // STATUS Register
    STATUS_REG.Address = 0x07;  // STATUS Register Address
    STATUS_REG.STDLAT  = 0x00;  // clear all status registers
    STATUS_REG.STD     = 0x00;
    STATUS_REG.UVLO    = 0x00;
    STATUS_REG.BPDF    = 0x00;
    STATUS_REG.APDF    = 0x00;
    STATUS_REG.BOCP    = 0x00;
    STATUS_REG.AOCP    = 0x00;
    STATUS_REG.OTS     = 0x00;

    //Initialize SPI
    EUSCI_B_SPI_initMasterParam param = {0};
    param.selectClockSource = EUSCI_B_SPI_CLOCKSOURCE_SMCLK;
    param.clockSourceFrequency = CS_getMCLK();
    param.desiredSpiClock = 500000;
    param.msbFirst = EUSCI_B_SPI_MSB_FIRST;
    param.clockPhase = EUSCI_B_SPI_PHASE_DATA_CAPTURED_ONFIRST_CHANGED_ON_NEXT;
    param.clockPolarity = EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_LOW;
    param.spiMode = EUSCI_B_SPI_3PIN;

    EUSCI_B_SPI_initMaster(EUSCI_B0_BASE, &param);
    EUSCI_B_SPI_enable(EUSCI_B0_BASE);

    DRV8711_writeAllRegisters();

    // only for debugging:
    // DRV8711_readAllRegisters();
    // DRV8711_resetFaults();

    // Disable SPI module after setting initial parameters to save power
    EUSCI_B_SPI_disable(EUSCI_B0_BASE);
}

void DRV8711_resetFaults(void)
{
    DRV8711_SPI_readWrite(0x70, 0x00);
}

void DRV8711_writeAllRegisters(void)
{
    unsigned char dataHi = 0x00;
    unsigned char dataLo = 0x00;

    // Write CTRL Register
    dataHi = REGWRITE | (CTRL_REG.Address << 4) | (CTRL_REG.DTIME << 2) | (CTRL_REG.ISGAIN);
    dataLo = (CTRL_REG.EXSTALL << 7) | (CTRL_REG.MODE << 3) | (CTRL_REG.RSTEP << 2) | (CTRL_REG.RDIR << 1) | (CTRL_REG.ENBL);
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write TORQUE Register
    dataHi = REGWRITE | (TORQUE_REG.Address << 4) | (TORQUE_REG.SMPLTH);
    dataLo = TORQUE_REG.TORQUE;
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write OFF Register
    dataHi = REGWRITE | (OFF_REG.Address << 4) | (OFF_REG.PWMMODE);
    dataLo = OFF_REG.TOFF;
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write BLANK Register
    dataHi = REGWRITE | (BLANK_REG.Address << 4) | (BLANK_REG.ABT);
    dataLo = BLANK_REG.TBLANK;
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write DECAY Register
    dataHi = REGWRITE | (DECAY_REG.Address << 4) | (DECAY_REG.DECMOD);
    dataLo = DECAY_REG.TDECAY;
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write STALL Register
    dataHi = REGWRITE | (STALL_REG.Address << 4) | (STALL_REG.VDIV << 2) | (STALL_REG.SDCNT);
    dataLo = STALL_REG.SDTHR;
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write DRIVE Register
    dataHi = REGWRITE | (DRIVE_REG.Address << 4) | (DRIVE_REG.IDRIVEP << 2) | (DRIVE_REG.IDRIVEN);
    dataLo = (DRIVE_REG.TDRIVEP << 6) | (DRIVE_REG.TDRIVEN << 4) | (DRIVE_REG.OCPDEG << 2) | (DRIVE_REG.OCPTH);
    DRV8711_SPI_readWrite(dataHi, dataLo);

    // Write STATUS Register
    dataHi = REGWRITE | (STATUS_REG.Address << 4);
    dataLo = (STATUS_REG.STDLAT << 7) | (STATUS_REG.STD << 6) | (STATUS_REG.UVLO << 5) | (STATUS_REG.BPDF << 4) | (STATUS_REG.APDF << 3) | (STATUS_REG.BOCP << 2) | (STATUS_REG.AOCP << 1) | (STATUS_REG.OTS);
    DRV8711_SPI_readWrite(dataHi, dataLo);
}

/*
 * Read registers just for debugging, results will not be written out
 */
void DRV8711_readAllRegisters(void)
{
    unsigned char dataHi = 0x00;
    const unsigned char dataLo = 0x00;
    volatile unsigned short readData = 0x00;

    // Read CTRL Register
    dataHi = REGREAD | (CTRL_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_CTRL_REG.Address       = CTRL_REG.Address;
    Test_CTRL_REG.DTIME         = ((readData >> 10) & 0x0003);
    Test_CTRL_REG.ISGAIN        = ((readData >> 8) & 0x0003);
    Test_CTRL_REG.EXSTALL       = ((readData >> 7) & 0x0001);
    Test_CTRL_REG.MODE          = ((readData >> 3) & 0x000F);
    Test_CTRL_REG.RSTEP         = ((readData >> 2) & 0x0001);
    Test_CTRL_REG.RDIR          = ((readData >> 1) & 0x0001);
    Test_CTRL_REG.ENBL          = ((readData >> 0) & 0x0001);

    // Read TORQUE Register
    dataHi = REGREAD | (TORQUE_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_TORQUE_REG.Address     = TORQUE_REG.Address;
    Test_TORQUE_REG.SMPLTH      = ((readData >> 8) & 0x0007);
    Test_TORQUE_REG.TORQUE      = ((readData >> 0) & 0x00FF);

    // Read OFF Register
    dataHi = REGREAD | (OFF_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_OFF_REG.Address        = OFF_REG.Address;
    Test_OFF_REG.PWMMODE        = ((readData >> 8) & 0x0001);
    Test_OFF_REG.TOFF           = ((readData >> 0) & 0x00FF);

    // Read BLANK Register
    dataHi = REGREAD | (BLANK_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_BLANK_REG.Address      = BLANK_REG.Address;
    Test_BLANK_REG.ABT          = ((readData >> 8) & 0x0001);
    Test_BLANK_REG.TBLANK       = ((readData >> 0) & 0x00FF);

    // Read DECAY Register
    dataHi = REGREAD | (DECAY_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_DECAY_REG.Address      = DECAY_REG.Address;
    Test_DECAY_REG.DECMOD       = ((readData >> 8) & 0x0007);
    Test_DECAY_REG.TDECAY       = ((readData >> 0) & 0x00FF);

    // Read STALL Register
    dataHi = REGREAD | (STALL_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_STALL_REG.Address      = STALL_REG.Address;
    Test_STALL_REG.VDIV         = ((readData >> 10) & 0x0003);
    Test_STALL_REG.SDCNT        = ((readData >> 8) & 0x0003);
    Test_STALL_REG.SDTHR        = ((readData >> 0) & 0x00FF);

    // Read DRIVE Register
    dataHi = REGREAD | (DRIVE_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_DRIVE_REG.Address      = DRIVE_REG.Address;
    Test_DRIVE_REG.IDRIVEP      = ((readData >> 10) & 0x0003);
    Test_DRIVE_REG.IDRIVEN      = ((readData >> 8) & 0x0003);
    Test_DRIVE_REG.TDRIVEP      = ((readData >> 6) & 0x0003);
    Test_DRIVE_REG.TDRIVEN      = ((readData >> 4) & 0x0003);
    Test_DRIVE_REG.OCPDEG       = ((readData >> 2) & 0x0003);
    Test_DRIVE_REG.OCPTH        = ((readData >> 0) & 0x0003);

    // Read STATUS Register
    dataHi = REGREAD | (STATUS_REG.Address << 4);
    readData = DRV8711_SPI_readWrite(dataHi, dataLo);
    Test_STATUS_REG.Address     = STATUS_REG.Address;
    Test_STATUS_REG.STDLAT      = ((readData >> 7) & 0x0001);
    Test_STATUS_REG.STD         = ((readData >> 6) & 0x0001);
    Test_STATUS_REG.UVLO        = ((readData >> 5) & 0x0001);
    Test_STATUS_REG.BPDF        = ((readData >> 4) & 0x0001);
    Test_STATUS_REG.APDF        = ((readData >> 3) & 0x0001);
    Test_STATUS_REG.BOCP        = ((readData >> 2) & 0x0001);
    Test_STATUS_REG.AOCP        = ((readData >> 1) & 0x0001);
    Test_STATUS_REG.OTS         = ((readData >> 0) & 0x0001);
}

unsigned short DRV8711_SPI_readWrite(unsigned char dataHi, unsigned char dataLo)
{
    unsigned short readData = 0;

    // set SCS line, i.e. alert DRV8711 to listen
    GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN7);

    // wait until SPI TX buffer is ready to take new data
    while(!EUSCI_B_SPI_getInterruptStatus(EUSCI_B0_BASE, EUSCI_B_SPI_TRANSMIT_INTERRUPT));
    // write and read high char
    EUSCI_B_SPI_transmitData(EUSCI_B0_BASE, dataHi);
    while(!EUSCI_B_SPI_getInterruptStatus(EUSCI_B0_BASE, EUSCI_B_SPI_RECEIVE_INTERRUPT));
    readData |= (EUSCI_B_SPI_receiveData(EUSCI_B0_BASE) << 8);

    // wait until SPI Tx buffer is ready to take new data
    while(!EUSCI_B_SPI_getInterruptStatus(EUSCI_B0_BASE, EUSCI_B_SPI_TRANSMIT_INTERRUPT));
    // write and read low char
    EUSCI_B_SPI_transmitData(EUSCI_B0_BASE, dataLo);
    while(!EUSCI_B_SPI_getInterruptStatus(EUSCI_B0_BASE, EUSCI_B_SPI_RECEIVE_INTERRUPT));
    readData |= EUSCI_B_SPI_receiveData(EUSCI_B0_BASE);

    // wait until SPI TX buffer is empty, i.e transmission completed
    while(!EUSCI_B_SPI_getInterruptStatus(EUSCI_B0_BASE, EUSCI_B_SPI_TRANSMIT_INTERRUPT));
    // clear SCS line to indicate that both high and low char were written
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN7);

    return readData;
}

void DRV8711_turnMotor(long steps)
{
    motorRunFlag = 1;

    DRV8711_exit_sleep();

    // Choose direction, DRV8711 DIR on 4.0
    if (steps < 0) GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN0); // DIR = clockwise
    else
    {
        GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0); // DIR = anti-clockwise
        steps += 20; // if turning forward, motor turns 10 extra steps forward and 10 backward at end of turn to release tension
    }

    step_counter = steps * microsteps;

    LCD_C_on(LCD_C_BASE);
    showFourDigitValueAndUnit(steps, 'S');

    // start timer B0 for step timing
    step_period = 400;

    Timer_B_initUpModeParam initUpParam_B0 =
    {
        TIMER_B_CLOCKSOURCE_ACLK,             // ACLK Clock Source = 32768Hz
        TIMER_B_CLOCKSOURCE_DIVIDER_1,        // ACLK/1 = 32768Hz
        step_period,                          // CCR0: initial micro step length ~= 3 ms
        TIMER_B_TBIE_INTERRUPT_DISABLE,       // Disable Timer interrupt
        TIMER_B_CCIE_CCR0_INTERRUPT_ENABLE ,  // Enable CCR0 interrupt
        TIMER_B_DO_CLEAR,                     // Clear value
        true                                  // Start Timer
    };
    Timer_B_initUpMode(TIMER_B0_BASE, &initUpParam_B0);
}

void DRV8711_stopMotor(void)
{
    Timer_B_clearCaptureCompareInterrupt(TIMER_B0_BASE, TIMER_B_CAPTURECOMPARE_REGISTER_0);
    Timer_B_stop(TIMER_B0_BASE);
    motorRunFlag = 0;
    DRV8711_enter_sleep();
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0); // clear dir pin
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN1); // clear step pin
    clearLCD();
    LCD_C_off(LCD_C_BASE);
}

void DRV8711_incrementStep(void)
{
    long stepsize = 10;
    if (motorRunFlag == 0) DRV8711_turnMotor(stepsize);
    else step_counter += stepsize * microsteps;
}
void DRV8711_decrementStep(void)
{
    long stepsize = -10;
    if (motorRunFlag == 0) DRV8711_turnMotor(stepsize);
    else step_counter += stepsize * microsteps;
}

void DRV8711_reset(void)
{
    // reset pin not accepted in sleep mode
    DRV8711_exit_sleep();

    // set the reset pin for a short time
    GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN0);
    __delay_cycles(100);  // ~0.1ms
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0);

    DRV8711_enter_sleep();
}

void DRV8711_exit_sleep(void)
{
    GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN3);
    __delay_cycles(1000);  //~1ms
}

void DRV8711_enter_sleep(void)
{
    GPIO_setOutputLowOnPin(GPIO_PORT_P9, GPIO_PIN3);
}

/*
 * Timer B0 Interrupt Service Routine
 * Used as stepper motor timer
 */
#pragma vector = TIMER0_B0_VECTOR
__interrupt void TIMER0_B0_ISR (void)
{
    // DRV8711 STEP on 4.1
    GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN1);

    // decrement absolute value of step counter
    if (step_counter < 0) step_counter++;
    else step_counter--;

    // update display, motor speed and direction every full step
    if ((step_counter % microsteps) == 0)
    {
        // always update display
        showFourDigitValueAndUnit(step_counter >> microstep_mode, 'S');

        // backward
        if (step_counter/microsteps < 0)
        {
            GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN0); // DIR = anticlockwise
            // Motor-Start-Ramp: Step period starts from 400 clock cycles and decreases every step down to 20
            if (step_period > 20)
            {
                step_period -= 4;
                Timer_B_setCompareValue(TIMER_B0_BASE, TIMER_B_CAPTURECOMPARE_REGISTER_0, step_period);
            }
        }

        // forward
        else if (step_counter/microsteps > 20)
        {
            GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0); // DIR = clockwise
            // Motor-Start-Ramp: Step period starts from 400 clock cycles and decreases every step down to 20
            if (step_period > 20)
            {
                step_period -= 4;
                Timer_B_setCompareValue(TIMER_B0_BASE, TIMER_B_CAPTURECOMPARE_REGISTER_0, step_period);
            }
        }

        // end of forward movement: slowly "overtwisting" 10 steps
        else if (step_counter/microsteps > 10)
        {
            GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0); // DIR = clockwise
            step_period = 400;  // slow
            Timer_B_setCompareValue(TIMER_B0_BASE, TIMER_B_CAPTURECOMPARE_REGISTER_0, step_period);
        }

        // slowly twist back 10 steps to release tension
        else if (step_counter/microsteps > 0)
        {
            GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN0); // DIR = anticlockwise
            step_period = 400;
            Timer_B_setCompareValue(TIMER_B0_BASE, TIMER_B_CAPTURECOMPARE_REGISTER_0, step_period);
        }
    }

    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN1);

    if (step_counter == 0) DRV8711_stopMotor();
}
