/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Helmholtz Centre for Environmental Research - UFZ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 * timer_mode.c
 *
 * Handles the Real Time Clock
 *
 *  Created on: 07.02.2017
 *      Author: G. Laube
 */

#include <driverlib.h>
#include "hal_LCD.h"
#include "timer_mode.h"
#include "hal_DRV8711.h"

void init_RTC(void)
{

    //Set current time to 01.01.2017 00:00:00
    initTime.Seconds = 0;
    initTime.Minutes = 0;
    initTime.Hours = 0;
    initTime.DayOfWeek = 0;
    initTime.DayOfMonth = 0;
    initTime.Month = 0;
    initTime.Year = 2017;

    RTC_C_initCalendar(RTC_C_BASE, &initTime, RTC_C_FORMAT_BINARY);

    //Setup Calendar Alarm to start
    //Note: Does not specify day of the week.
    startParam.minutesAlarm = st_minutes;
    startParam.hoursAlarm = st_hours;
    startParam.dayOfWeekAlarm = RTC_C_ALARMCONDITION_OFF;
    startParam.dayOfMonthAlarm = st_days;

    // This will be set by the RTC update function
    updateParam.minutesAlarm = minutes;
    updateParam.hoursAlarm = hours;
    updateParam.dayOfWeekAlarm = RTC_C_ALARMCONDITION_OFF;
    updateParam.dayOfMonthAlarm = days;

    // if a "start in ..." time is given, initialize RTC alarm with waiting time
    if (st_minutes+st_hours+st_days != 0)
    {
        // On the first call of the update function, motor doesn't need to turn.
        startInFlag = 1;
        RTC_C_configureCalendarAlarm(RTC_C_BASE, &startParam);
    }
    // else, start immediately, i.e. set regular interval alarm
    else
    {
        startInFlag = 0;
        RTC_C_configureCalendarAlarm(RTC_C_BASE, &updateParam);
    }

    // reset bottle counter, will fill 18 bottles
    bottleCounter = 0;

    //Enable interrupts for the Calendar alarm
    RTC_C_enableInterrupt(RTC_C_BASE, RTCAIE);
    //Start RTC Clock
    RTC_C_startClock(RTC_C_BASE);
}

void update_RTC(void)
{
    //Disable interrupts for the Calendar alarm
    RTC_C_disableInterrupt(RTC_C_BASE, RTCAIE);

    //Set back to 01.01.2017 00:00:00
    //Will automatically hold RTC for save writing into the calendar registers
    RTC_C_initCalendar(RTC_C_BASE, &initTime, RTC_C_FORMAT_BINARY);

    //Set alarm to updateDays and updateHours
    RTC_C_configureCalendarAlarm(RTC_C_BASE, &updateParam);

    //Enable interrupts for the Calendar alarm
    RTC_C_enableInterrupt(RTC_C_BASE, RTCAIE);

    //Start RTC Clock
    RTC_C_startClock(RTC_C_BASE);
}

#pragma vector=RTC_VECTOR
__interrupt void RTC_ISR(void)
{
    switch(__even_in_range(RTCIV, RTCIV_RT1PSIFG))
    {
    case RTCIV_NONE: break;         // No interrupts
    case RTCIV_RTCOFIFG: break;     /* RTC Osc fault: RTCOFIFG */
    case RTCIV_RTCRDYIFG: break;    /* RTC ready: RTCRDYIFG */
    case RTCIV_RTCTEVIFG: break;    /* RTC interval timer: RTCTEVIFG */
    case RTCIV_RTCAIFG:             /* RTC user alarm: RTCAIFG */
        // start new interval
        update_RTC();

        if (startInFlag)
        {
            startInFlag = 0;
            LCD_C_on(LCD_C_BASE);
            displayScrollText("START OF FIRST SAMPLING INTERVAL");
            LCD_C_off(LCD_C_BASE);
        }
        else if (bottleCounter < 17)
        {
            LCD_C_on(LCD_C_BASE);
            displayScrollText("TURNING MOTOR");
            LCD_C_off(LCD_C_BASE);
            DRV8711_turnMotor(853); // NEMA23: turn motor by 853 steps = 20 deg
            //DRV8711_turnMotor(1107); //NEMA11: turn motor by 1106 steps = 20 deg, 1107 seems to work out better
            bottleCounter++;
        }
        else
        {
            LCD_C_on(LCD_C_BASE);
            displayScrollText("SAMPLING COMPLETED      STANDBY");
            LCD_C_off(LCD_C_BASE);
            RTC_C_holdClock(RTC_C_BASE);
        }
        break;
    case RTCIV_RT0PSIFG: break;     /* RTC prescaler 0: RT0PSIFG */
    case RTCIV_RT1PSIFG: break;     /* RTC prescaler 1: RT1PSIFG */

    default: break;
    }
}
